"""cctek package.

Defines:
Card,
InvalidIIN,
InvalidTrackData,
Parser,
bin_checker,
build_iin_path,
get_cc_type,
load_iin,
luhn_checker,
random_amex_card,
random_carteblanche_diners_card,
random_discover_card,
random_enroute_card,
random_euro_mastercard,
random_jcb_card,
random_visa_card,
validate_cvv_format,
"""

from .magnet import (
    Card,
    InvalidTrackData,
    Parser,
)
from .dataloader import (
    InvalidIIN,
    build_iin_path,
    load_iin,
)
from .cardchecker import (
    bin_checker,
    get_cc_type,
    luhn_checker,
    random_amex_card,
    random_carteblanche_diners_card,
    random_discover_card,
    random_enroute_card,
    random_euro_mastercard,
    random_jcb_card,
    random_visa_card,
    validate_cvv_format,
)

__all__ = [
    "Card",
    "InvalidIIN",
    "InvalidTrackData",
    "Parser",
    "bin_checker",
    "build_iin_path",
    "get_cc_type",
    "load_iin",
    "luhn_checker",
    "random_amex_card",
    "random_carteblanche_diners_card",
    "random_discover_card",
    "random_enroute_card",
    "random_euro_mastercard",
    "random_jcb_card",
    "random_visa_card",
    "validate_cvv_format",
]

__version__ = "0.0.6"



"""Console Scripts Entry Points For cctek Distribution."""
import cctek
import argparse
import yaml


def luhn_check(args):
    """Run Luhn Algorithm checks on the card numbers."""
    for c in args.cards:
        res = {str(c): cctek.luhn_checker(c)}
        print(yaml.dump(res))
    return


def lookup_bin(args):
    """Lookup Issuer Identification Numbers."""
    for b in args.bins:
        res = cctek.bin_checker(b)
        print(yaml.dump(res))
    return


def random_card(args):
    """Generate a random credit card number of the specified type."""
    fmap = {
        "amex": cctek.random_amex_card,
        "diners": cctek.random_carteblanche_diners_card,
        "discover": cctek.random_discover_card,
        "enroute": cctek.random_enroute_card,
        "mastercard": cctek.random_euro_mastercard,
        "jcb": cctek.random_jcb_card,
        "visa": cctek.random_visa_card,
    }
    print(fmap.get(args.card_type)())
    return


def main():
    """Run Main Command Executor."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--version", action="version", version=f"cctek {cctek.__version__}"
    )
    subparsers = parser.add_subparsers()
    # random card subparser card_type
    random_card_parser = subparsers.add_parser(
        "random_card",
        aliases=[
            "rc",
        ],
        help="""Run Luhn Algorithm checks on the card numbers.""",
    )
    random_card_parser.set_defaults(func=random_card)
    random_card_parser.add_argument(
        "card_type",
        choices=[
            "amex",
            "diners",
            "discover",
            "enroute",
            "jcb",
            "mastercard",
            "visa",
        ],
    )
    # luhn checker subparser cards
    luhn_parser = subparsers.add_parser(
        "luhn",
        aliases=[
            "ln",
        ],
        help="""Run Luhn Algorithm checks on the card numbers.""",
    )
    luhn_parser.set_defaults(func=luhn_check)
    luhn_parser.add_argument(
        "cards",
        help="""Credit Card Numbers or Bank/Issuer Identification Numbers to Lookup.""",
        nargs="+",
        metavar="credit_card_number(s)",
    )
    # lookup bin subparser bins
    lookup_bin_parser = subparsers.add_parser(
        "lookup",
        aliases=[
            "lk",
        ],
        help="""Lookup Issuer Identification Numbers.""",
    )
    lookup_bin_parser.set_defaults(func=lookup_bin)
    lookup_bin_parser.add_argument(
        "bins",
        help="""Credit Card Number or Bank/Issuer Identification Number(first 6
        digits) to Lookup.""",
        nargs="+",
        metavar="BIN(s)",
    )
    # parse the args
    args = parser.parse_args()
    args.func(args)
    return



#!/usr/bin/env python
"""Data Loader Module.

Utilities to load Bank Indentification Numbers, more recently referred to as
Issuer Identification Numbers from the YAML format into python Objects.
"""
from yaml import load  # ,dump

# from pprint import pprint
import json
import os

try:
    from yaml import CLoader as Loader  # ,CDumper as Dumper
except ImportError:
    from yaml import Loader  # ,Dumper


CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))


class InvalidIIN(Exception):
    """Exception Handler for Too Short IIN|BIN Inputs."""

    pass


def build_iin_path(iin):
    """Return the Appropriate Path to the specified IIN|BIN."""
    if isinstance(iin, int):
        iin = str(iin)
    if not len(iin) >= 4:
        raise InvalidIIN("Issuer Identification Number Too Short < 6")
    return os.path.join(CURRENT_DIR, f"data/{iin[0]}/{iin[:4]}.yml")


def load_iin(iin):
    """Return a Python Object from the Loaded Yaml Data File."""
    path = build_iin_path(iin)
    with open(path, "r") as fl:
        document = load(fl, Loader=Loader)

    return document


if __name__ == "__main__":
    import sys

    if not len(sys.argv) > 1:
        testc = "554213561141250"
        testcc = "4542"
        print(json.dumps(load_iin(testcc)))
    else:
        testcc = sys.argv[1]
        if len(testcc) >= 6:
            print(json.dumps(load_iin(testcc).get(testcc[:6])))
        else:
            print(json.dumps(load_iin(testcc)))



#!/usr/bin/env python
"""Magnetic Card Parser."""
import string
import re
import yaml


class InvalidTrackData(Exception):
    """InvalidTrackData Exception."""

    pass


class Parser:
    """Track Parser."""

    TRACKS = {
        "1": re.compile(
            r"%(?P<format>[A-Z])(?P<pan>[0-9]{1,19})\^(?P<name>[^^]*)\^\s?(?P<expiration>\d{4}|\^)(?P<service_code>\d{3}|\^)(?P<discretionary_data>[^\?]*)\?"
        ),
        "2": re.compile(
            r";(?P<pan>[0-9]{1,19})=(?P<expiration>\d{4}|=)(?P<service_code>\d{3}|=)(?P<discretionary_data>[^\?]*)\?.?"
        ),
        "emv": re.compile(
            r";(?P<pan>[0-9 ]{1,19})(d|D)(?P<expiration>\d{4}|(d|D))(?P<service_code>\d{3}|(d|D))(?P<discretionary_data>[^\?fF]*)(f|F)?0?.*"
        ),
    }

    @staticmethod
    def parse(track_data):
        """Parse track_data and return attributes."""
        for k, v in Parser.TRACKS.items():
            match = v.match(track_data)
            if match:
                attributes = {}
                attributes["track_format"] = k
                attributes["pan"] = match.group("pan")
                attributes["discretionary_data"] = match.group(
                    "discretionary_data")
                if k == "1":
                    attributes["format"] = match.group("format")
                    attributes["name"] = match.group("name")
                    if match.group("expiration") == "^":
                        attributes["expiration"] = "00", "00"
                    else:
                        attributes["expiration"] = match.group(
                            "expiration"
                        )  # if == ^ none
                    if match.group("service_code") == "^":
                        attributes["service_code"] = "0", "0", "0"
                    else:
                        attributes["service_code"] = match.group(
                            "service_code"
                        )  # if == ^ none
                elif k == "2" or k == "emv":
                    if match.group("expiration") == "=":
                        attributes["expiration"] = "00", "00"
                    else:
                        attributes["expiration"] = match.group(
                            "expiration"
                        )  # if == = none
                    if match.group("service_code") == "=":
                        attributes["service_code"] = "0", "0", "0"
                    else:
                        attributes["service_code"] = match.group(
                            "service_code"
                        )  # if == = none
                else:
                    pass
                return attributes
            else:
                # raise InvalidTrackData(
                #     f"Track Data: {track_data} is not Valid.")
                pass


class Card:
    """
    Card Class.

    attr_accessor :allowed_services, :authorization_processing,
    :discretionary_data, :expiration_year, :expiration_month, :first_name,
    :format, :initial, :interchange, :last_name, :number, :pin_requirements,
    :technology, :title, :track_format."""

    ALLOWED_SERVICES = {
        "0": "no_restrictions",
        "1": "no_restrictions",
        "2": "goods_and_services_only",
        "3": "atm_only",
        "4": "cash_only",
        "5": "goods_and_services_only",
        "6": "no_restrictions",
        "7": "goods_and_services_only",
    }
    AUTHORIZATION_PROCESSING = {
        "0": "normal",
        "2": "by_issuer",
        "4": "by_issuer_unless_explicit_agreement",
    }
    FORMAT = {
        "A": "reserved",
        "B": "bank",
    }
    FORMAT.update({c: "available" for c in string.ascii_uppercase[2:13]})
    FORMAT.update({c: "available" for c in string.ascii_uppercase[13:]})
    INTERCHANGE = {
        "1": "international",
        "2": "international",
        "5": "national",
        "6": "national",
        "7": "private",
        "9": "test",
    }
    PIN_REQUIREMENTS = {
        "0": "pin_required",
        "3": "pin_required",
        "5": "pin_required",
        "6": "prompt_for_pin_if_ped_present",
    }
    TECHNOLOGY = {
        "2": "integrated_circuit_card",
        "6": "integrated_circuit_card",
    }

    def __init__(self):
        self.allowed_services = None
        self.authorization_processing = None
        self.discretionary_data = None
        self.expiration_month = None
        self.expiration_year = None
        self.first_name = None
        self.format = None
        self.interchange = None
        self.last_name = None
        self.number = None
        self.pin_requirements = None
        self.technology = None
        self.title = None
        self.track_format = None

    @staticmethod
    def parse(track_data, parser=Parser()):
        attributes = parser.parse(track_data)
        if not attributes:
            return Card()
        # print(yaml.dump(attributes))
        position1, position2, position3 = attributes.get("service_code")
        year, month = attributes.get("expiration")[
            :2], attributes.get("expiration")[2:]
        if attributes.get("name"):
            title, first_name, initial, last_name = Card.parse_name(
                attributes.get("name", "")
            )
        else:
            title, first_name, initial, last_name = Card.parse_name("")
        card = Card()
        card.allowed_services = Card.ALLOWED_SERVICES.get(position3)
        card.authorization_processing = Card.AUTHORIZATION_PROCESSING.get(
            position2)
        card.discretionary_data = attributes.get("discretionary_data")
        card.expiration_month = month
        card.expiration_year = year
        card.first_name = first_name
        card.format = Card.FORMAT.get(attributes.get("format"))
        card.interchange = Card.INTERCHANGE.get(position1)
        card.last_name = last_name
        card.number = attributes.get("pan").replace(" ", "")
        card.pin_requirements = Card.PIN_REQUIREMENTS.get(position3)
        card.technology = Card.TECHNOLOGY.get(position1)
        card.title = title
        card.track_format = attributes.get("track_format")
        return card

    @staticmethod
    def parse_name(name):
        title, first, initial, last = "", "", "", ""
        try:
            last, first = name.split("/")[0:2]
        except ValueError:
            first = name
            last = ""
        if first:
            try:
                first, initial = first.split(" ")
            except ValueError:
                initial = ""
        if initial:
            try:
                initial, title = initial.split(".")[0:2]
            except ValueError:
                title = ""
        return (title, first, initial, last)

    @property
    def atm_only(self):
        return self.allowed_services == "atm_only"

    @property
    def cash_only(self):
        return self.allowed_services == "cash_only"

    @property
    def goods_and_services_only(self):
        return self.allowed_services == "goods_and_services_only"

    @property
    def no_service_restrictions(self):
        return self.allowed_services == "no_restrictions"

    @property
    def integrated_circuit_card(self):
        return self.technology == "integrated_circuit_card"

    @property
    def international(self):
        return self.interchange == "international"

    @property
    def test(self):
        return self.interchange == "test"

    @property
    def national(self):
        return self.interchange == "national"

    @property
    def private(self):
        return self.interchange == "private"

    @property
    def pin_required(self):
        return self.pin_requirements == "pin_required"

    @property
    def prompt_for_pin_if_ped_present(self):
        return self.pin_requirements == "prompt_for_pin_if_ped_present"

    @property
    def process_by_issuer(self):
        return self.authorization_processing == "by_issuer"

    @property
    def process_by_issuer_unless_explicit_agreement(self):
        return self.authorization_processing == "by_issuer_unless_explicit_agreement"

    @property
    def process_normally(self):
        return self.authorization_processing == "normal"

    @property
    def as_dict(self):
        return {
            "allowed_services": self.allowed_services,
            "authorization_processing": self.authorization_processing,
            "discretionary_data": self.discretionary_data,
            "expiration_month": self.expiration_month,
            "expiration_year": self.expiration_year,
            "first_name": self.first_name,
            "format": self.format,
            "interchange": self.interchange,
            "last_name": self.last_name,
            "number": self.number,
            "pin_requirements": self.pin_requirements,
            "technology": self.technology,
            "title": self.title,
            "track_format": self.track_format,
        }

    def __str__(self):
        return yaml.dump(self.as_dict)

    def __repr__(self):
        return f"Card: {self.as_dict}"


if __name__ == "__main__":
    test_track1_data = (
        "%B5350290149345177^FATEHI/SUALEH^16042010000000000000000000000000000567001000?"
    )
    test_track2_data = ";5350290149345177=16042010000056700100?"
    card = Card.parse(test_track1_data)
    print(card)
    pass



#!/usr/bin/env python
"""CreditCardChecker."""

import re
import random
from . import dataloader


def bin_checker(card):
    """The first 6 digit of a card is known as BIN Number.


    BIN stands for Bank Identification Number.
    It's a 6 digit code which identifies the financial institution that issued the Card to the card holder.
    It also referred as IIN (Issuer Identification Number).
    BIN can be used to identify issuing financial institution of a Credit Card, Debit Card, Prepaid Card and rewards Card.
    Credit & Debit Card Number Format
    As per ISO/IEC 7812, the length of Credit & Debit Card number can be upto 19 digits.
    PQQQQRRRRRRRRRRRS

    P - Major industry Identifier

    Q - Issuing Financial Institution identifier

    R - Account Identifier up to 12 digits

    S - Checksum to validate Card Number.

    Where PQQQQQ combined are known as BIN number or IIN code.

    Uses of BIN Number
    It is commonly used by merchants to identify the card type and issuing bank of the Credit Card.

    It also helps in preventing Fraud. In case the Billing Address and the issuing country of the card is different, it may be a fraudulent transaction. With BIN search they can easily identify such transactions.
    """
    return dataloader.load_iin(card).get(card[:6])


def luhn_checker(card, debug=False):
    """Validate whether credit card number satisfies luhn's algorithm.

    set debug to True to print debugging details like calculated checksum digit,
    PAN(Primary Account Number) and Whole Card String.
    Only when a valid checksum is validated do we output the whole card: else
    just calculated checksum and expected checksum.
    Return True else False
    """
    odd_digits = card[0:-1:2]
    even_digits = card[1:-1:2]
    # Double odd_digits
    dbl_odd_digits = [int(x) * 2 for x in odd_digits]
    # subtract 9 from dbl_digits if digit is >10 else remain same
    dbl_less_nine = [j if j < 10 else j - 9 for j in dbl_odd_digits]
    even_digits_as_Z = [int(x) for x in even_digits]
    # calculate checksum using 10-( sum(of even_digits+dbl_less_nine) % 10)
    checksum = 10 - ((sum(even_digits_as_Z) + sum(dbl_less_nine)) % 10)
    # Get The PAN (Primary Account Number)
    # from the 7th digit of the card number
    # If the Checksum is equal to the last digit of card, then it satisfies
    # luhns algorithm so we return true else false.
    if str(checksum) == card[-1]:
        if debug:
            PAN = card[6:]  # Not used in calculation FYI
            print(f"Card:\t{card}")
            print(f"PAN:\t{PAN}")
            print(
                f"Calculated Checksum:\t{checksum} \tExpected Checksum:{card[-1]}")
        return True
    else:
        if debug:
            print(
                f"Calculated Checksum:\t{checksum} \tExpected Checksum:{card[-1]}")
        return False


def random_euro_mastercard():
    # 5[1-5][0-9]{14}
    a = range(1, 6)
    b = range(0, 10)
    prefix = f"5{random.choice(a)}"
    suffix = "".join([str(random.choice(b)) for i in range(14)])
    return prefix + suffix


# 4([0-9]{12}|[0-9]{15})
def random_visa_card():
    b = range(0, 10)
    prefix = "4"
    suffix = random.choice(
        [
            "".join([str(random.choice(b)) for i in range(12)]),
            "".join([str(random.choice(b)) for i in range(15)]),
        ]
    )
    return prefix + suffix


# 3[47][0-9]{13}
def random_amex_card():
    a = [4, 7]
    b = range(0, 10)
    prefix = f"3{random.choice(a)}"
    suffix = "".join([str(random.choice(b)) for i in range(13)])
    return prefix + suffix


# 3(0[0-5][0-9]{11}|[68][0-9]{12})
def random_carteblanche_diners_card():
    prefix = "3"
    b = range(0, 10)
    suffix = random.choice(
        [
            f"0{random.choice(range(6))}"
            + "".join([str(random.choice(b)) for i in range(11)]),
            f"{random.choice([6,8])}"
            + "".join([str(random.choice(b)) for i in range(12)]),
        ]
    )
    return prefix + suffix


# 6011[0-9]{12}
def random_discover_card():
    b = range(0, 10)
    return "6011" + "".join(str(random.choice(b)) for i in range(12))


# (3[0-9]{15}|(2131|1800)[0-9]{11})
def random_jcb_card():
    b = range(0, 10)
    return random.choice(
        [
            "3" + "".join(str(random.choice(b)) for i in range(15)),
            "2131" + "".join(str(random.choice(b)) for i in range(11)),
            "1800" + "".join(str(random.choice(b)) for i in range(11)),
        ]
    )


# 2(014|149)[0-9]{11}
def random_enroute_card():
    b = range(0, 10)
    return random.choice(
        [
            "2014" + "".join(str(random.choice(b)) for i in range(11)),
            "2149" + "".join(str(random.choice(b)) for i in range(11)),
        ]
    )


def get_cc_type(pattern: str) -> list:
    if re.match("5[1-5][0-9]{14}", pattern):
        return ["MASTERCARD", "EUROCARD", "EUROCARD/MASTERCARD"]

    if re.match("4([0-9]{12}|[0-9]{15})", pattern):
        return [
            "VISA",
        ]

    if re.match("3[47][0-9]{13}", pattern):
        return ["AMEX", "AMERICANEXPRESS", "AMERICAN EXPRESS"]

    if re.match("3(0[0-5][0-9]{11}|[68][0-9]{12})", pattern):
        return ["DINERS", "DINERSCLUB", "DINERS CLUB", "CARTEBLANCHE", "CARTE BLANCHE"]

    if re.match("6011[0-9]{12}", pattern):
        return [
            "DISCOVER",
        ]

    if re.match("(3[0-9]{15}|(2131|1800)[0-9]{11})", pattern):
        return [
            "JCB",
        ]

    if re.match("2(014|149)[0-9]{11}", pattern):
        return [
            "ENROUTE",
        ]
    return [
        "UKNOWN",
    ]


def validate_cvv_format(cvv: str, cardType: str):
    """Validate a card verification value format.

    * This method only checks for the format. It doesn't
    * validate that the value is the one on the card.
    *
    * CVV is also known as
    *  - CVV2 Card Validation Value 2 (Visa)
    *  - CVC  Card Validation Code (MasterCard)
    *  - CID  Card Identification (American Express and Discover)
    *  - CIN  Card Identification Number
    *  - CSC  Card Security Code
    *
    * Important information regarding CVV:
    *    If you happen to have to store credit card information, you must
    *    NOT retain the CVV after transaction is complete. Usually this
    *    means you cannot store it in a database, not even in an encrypted
    *    form.
    *
    * This method returns FALSE for card types that don't support CVV.
    *
    * param: string cvv      value to verify
    * param: string cardType type/brand of card (case insensitive)
    *               "MasterCard", "Visa", "AMEX", "AmericanExpress",
    *               "American Express", "Discover", "Eurocard/MasterCard",
    *               "Eurocard"
    *
    * return bool   TRUE if format is correct, FALSE otherwise
    """
    if cardType.upper() in [
        "MASTERCARD",
        "EUROCARD",
        "EUROCARD/MASTERCARD",
        "VISA",
        "DISCOVER",
    ]:
        digits = 3

    if cardType.upper() in ["AMEX", "AMERICANEXPRESS", "AMERICAN EXPRESS"]:
        digits = 4

    if len(cvv) == digits and cvv.isdigit():
        return True

    return False
